<?php

  /*

   Expiring Cache

   -- Copyright --
   Parts Copyright 2004, by Paul Burdick (reedmaniac@pmachine.com)
   Parts Copyright 2004, by James McDuffie (ttq3sxl02@sneakemail.com)

   -- License --
   GNU GPL

   -- Description --

   Caches generated on disk and regenerates it if it is past its
   expiration date.

   -- Example Usage --

   -- History --

   0.1 - Initial version with code extracted from Netflix Queue class

  */

define(EXPIRING_CACHE_VERSION, '0.1'); 

Class ExpiringCache {

  // Period between cache refreshes (in minutes)
  var $cache_refresh = 1440; # Default is to refresh every day

  // Files for storing cache information
  var $data_cache_file   = '';
  var $time_cache_file   = '';

  // Whether or not the data needs to be regenerated
  var $generation_needed = true;
  
  //-------------------------------------
  //  Constructor
  //-------------------------------------

  function ExpiringCache($cache_base_filename, $cache_refresh = 1440) {
   	
    // -------------------------------
    //  Set class variables
    // -------------------------------

    $this->cache_refresh   = $cache_refresh;
    $this->data_cache_file = $cache_base_filename . "-data";
    $this->time_cache_file = $cache_base_filename . "-time";

    // Check cache data and load it if does not need generation
    $this->check_cache_data();
        
  }
  // END

  //-------------------------------------
  // Returns retrieved cache data
  //-------------------------------------

  function get_cache_data() {
    return $this->cache_data;
  }

  //-------------------------------------
  // Returns whether or not data generation is needed
  //-------------------------------------

  function generation_needed() {
    return $this->generation_needed;
  }

  //-------------------------------------
  // Stores data into cache file
  //-------------------------------------

  function store_data_into_cache($data) {

    // Write Cache Time File
    if ($fp = @fopen($this->time_cache_file, 'wb')) {
      flock($fp, LOCK_SH);
      fwrite($fp, time());
      flock($fp, LOCK_UN);
      fclose($fp);
    }
    			
    // Write Cache File
    if ($fp = @fopen($this->data_cache_file, 'wb')) {
      flock($fp, LOCK_SH);
      fwrite($fp, $data);
      flock($fp, LOCK_UN);
      fclose($fp);
    }

  }
	
  //-------------------------------------
  // Check for expired cache
  //-------------------------------------   

  function check_cache_data() {
    
    $self->cache_data = '';

    // --------------------------
    // Check Cache
    // --------------------------
    if ( ! is_dir(dirname($this->data_cache_file))) {
      trigger_error( "Cache directory '" . dirname($this->data_cache_file) . 
		     "' does not exist" );
    }

    if ( !file_exists($this->data_cache_file) ||  
	 !file_exists($this->time_cache_file)) {

      $this->generation_needed = true;
      return;

    } else {
      $fp = @fopen($this->time_cache_file, 'r+b');
      $sp = @fopen($this->data_cache_file, 'r+b');
        	
      if ( ! is_resource($fp) ||  ! is_resource($sp)) {

	trigger_error( "Time or data cache file not opened succesfully: " .
		       $this->time_cache_file . ", " . $this->data_cache_file );

      } else {

	flock($fp, LOCK_SH);
	$timestamp = trim(@fread($fp, filesize($this->time_cache_file)));
	flock($fp, LOCK_UN);
	fclose($fp);
        
	if (time() > ($timestamp + ($this->cache_refresh * 60))) {

	  $this->generation_needed = true;
	  return;

	} else {
	  flock($sp, LOCK_SH);
	  $this->cache_data = trim(@fread($sp, filesize($this->data_cache_file)));
	  flock($sp, LOCK_UN);
	  fclose($sp);
	}
      }
    }

    $this->generation_needed = false;

  }
  // END

} // END ExpiringCache class
?>